# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Gdpr Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class DtoPlatformAccountClosureClientResponse(Model):
    """Dto platform account closure client response (dto.PlatformAccountClosureClientResponse)

    Properties:
        namespace: (namespace) REQUIRED str

        platform: (platform) REQUIRED str

        updated_at: (updatedAt) REQUIRED int

        bp_cert_expire_at: (bpCertExpireAt) OPTIONAL int

        bp_cert_file_name: (bpCertFileName) OPTIONAL str

        client_id: (clientId) OPTIONAL str

        publisher_key: (publisherKey) OPTIONAL str

        sandbox_id: (sandboxId) OPTIONAL str

        secret: (secret) OPTIONAL str
    """

    # region fields

    namespace: str  # REQUIRED
    platform: str  # REQUIRED
    updated_at: int  # REQUIRED
    bp_cert_expire_at: int  # OPTIONAL
    bp_cert_file_name: str  # OPTIONAL
    client_id: str  # OPTIONAL
    publisher_key: str  # OPTIONAL
    sandbox_id: str  # OPTIONAL
    secret: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_namespace(self, value: str) -> DtoPlatformAccountClosureClientResponse:
        self.namespace = value
        return self

    def with_platform(self, value: str) -> DtoPlatformAccountClosureClientResponse:
        self.platform = value
        return self

    def with_updated_at(self, value: int) -> DtoPlatformAccountClosureClientResponse:
        self.updated_at = value
        return self

    def with_bp_cert_expire_at(
        self, value: int
    ) -> DtoPlatformAccountClosureClientResponse:
        self.bp_cert_expire_at = value
        return self

    def with_bp_cert_file_name(
        self, value: str
    ) -> DtoPlatformAccountClosureClientResponse:
        self.bp_cert_file_name = value
        return self

    def with_client_id(self, value: str) -> DtoPlatformAccountClosureClientResponse:
        self.client_id = value
        return self

    def with_publisher_key(self, value: str) -> DtoPlatformAccountClosureClientResponse:
        self.publisher_key = value
        return self

    def with_sandbox_id(self, value: str) -> DtoPlatformAccountClosureClientResponse:
        self.sandbox_id = value
        return self

    def with_secret(self, value: str) -> DtoPlatformAccountClosureClientResponse:
        self.secret = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "platform"):
            result["platform"] = str(self.platform)
        elif include_empty:
            result["platform"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = int(self.updated_at)
        elif include_empty:
            result["updatedAt"] = 0
        if hasattr(self, "bp_cert_expire_at"):
            result["bpCertExpireAt"] = int(self.bp_cert_expire_at)
        elif include_empty:
            result["bpCertExpireAt"] = 0
        if hasattr(self, "bp_cert_file_name"):
            result["bpCertFileName"] = str(self.bp_cert_file_name)
        elif include_empty:
            result["bpCertFileName"] = ""
        if hasattr(self, "client_id"):
            result["clientId"] = str(self.client_id)
        elif include_empty:
            result["clientId"] = ""
        if hasattr(self, "publisher_key"):
            result["publisherKey"] = str(self.publisher_key)
        elif include_empty:
            result["publisherKey"] = ""
        if hasattr(self, "sandbox_id"):
            result["sandboxId"] = str(self.sandbox_id)
        elif include_empty:
            result["sandboxId"] = ""
        if hasattr(self, "secret"):
            result["secret"] = str(self.secret)
        elif include_empty:
            result["secret"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        namespace: str,
        platform: str,
        updated_at: int,
        bp_cert_expire_at: Optional[int] = None,
        bp_cert_file_name: Optional[str] = None,
        client_id: Optional[str] = None,
        publisher_key: Optional[str] = None,
        sandbox_id: Optional[str] = None,
        secret: Optional[str] = None,
        **kwargs,
    ) -> DtoPlatformAccountClosureClientResponse:
        instance = cls()
        instance.namespace = namespace
        instance.platform = platform
        instance.updated_at = updated_at
        if bp_cert_expire_at is not None:
            instance.bp_cert_expire_at = bp_cert_expire_at
        if bp_cert_file_name is not None:
            instance.bp_cert_file_name = bp_cert_file_name
        if client_id is not None:
            instance.client_id = client_id
        if publisher_key is not None:
            instance.publisher_key = publisher_key
        if sandbox_id is not None:
            instance.sandbox_id = sandbox_id
        if secret is not None:
            instance.secret = secret
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> DtoPlatformAccountClosureClientResponse:
        instance = cls()
        if not dict_:
            return instance
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "platform" in dict_ and dict_["platform"] is not None:
            instance.platform = str(dict_["platform"])
        elif include_empty:
            instance.platform = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = int(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = 0
        if "bpCertExpireAt" in dict_ and dict_["bpCertExpireAt"] is not None:
            instance.bp_cert_expire_at = int(dict_["bpCertExpireAt"])
        elif include_empty:
            instance.bp_cert_expire_at = 0
        if "bpCertFileName" in dict_ and dict_["bpCertFileName"] is not None:
            instance.bp_cert_file_name = str(dict_["bpCertFileName"])
        elif include_empty:
            instance.bp_cert_file_name = ""
        if "clientId" in dict_ and dict_["clientId"] is not None:
            instance.client_id = str(dict_["clientId"])
        elif include_empty:
            instance.client_id = ""
        if "publisherKey" in dict_ and dict_["publisherKey"] is not None:
            instance.publisher_key = str(dict_["publisherKey"])
        elif include_empty:
            instance.publisher_key = ""
        if "sandboxId" in dict_ and dict_["sandboxId"] is not None:
            instance.sandbox_id = str(dict_["sandboxId"])
        elif include_empty:
            instance.sandbox_id = ""
        if "secret" in dict_ and dict_["secret"] is not None:
            instance.secret = str(dict_["secret"])
        elif include_empty:
            instance.secret = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, DtoPlatformAccountClosureClientResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[DtoPlatformAccountClosureClientResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        DtoPlatformAccountClosureClientResponse,
        List[DtoPlatformAccountClosureClientResponse],
        Dict[Any, DtoPlatformAccountClosureClientResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "namespace": "namespace",
            "platform": "platform",
            "updatedAt": "updated_at",
            "bpCertExpireAt": "bp_cert_expire_at",
            "bpCertFileName": "bp_cert_file_name",
            "clientId": "client_id",
            "publisherKey": "publisher_key",
            "sandboxId": "sandbox_id",
            "secret": "secret",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "namespace": True,
            "platform": True,
            "updatedAt": True,
            "bpCertExpireAt": False,
            "bpCertFileName": False,
            "clientId": False,
            "publisherKey": False,
            "sandboxId": False,
            "secret": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsContentLikeRequest
from ..models import ModelsContentLikeResponse
from ..models import ModelsPaginatedContentDownloadResponse
from ..models import ResponseError

from ..operations.public_like_legacy import GetLikedContent
from ..operations.public_like_legacy import UpdateContentLikeStatus


@same_doc_as(GetLikedContent)
def get_liked_content(
    isofficial: Optional[bool] = None,
    limit: Optional[int] = None,
    name: Optional[str] = None,
    offset: Optional[int] = None,
    orderby: Optional[str] = None,
    sortby: Optional[str] = None,
    subtype: Optional[str] = None,
    tags: Optional[List[str]] = None,
    type_: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get liked contents (GetLikedContent)

    For advance tag filtering supports & as AND operator and | as OR operator and parentheses ( ) for priority. e.g:


    *tags=red*
    *tags=red&animal;*
    *tags=red|animal*
    *tags=red&animal;|wild*
    *tags=red&(animal|wild)*

    The precedence of logical operator is AND > OR, so if no parentheses, AND logical operator will be executed first.

    Allowed character for operand: alphanumeric, underscore _ and dash -

    Allowed character for operator: & | ( )


    **Please note that value of tags query param should be URL encoded**

    Properties:
        url: /ugc/v1/public/namespaces/{namespace}/contents/liked

        method: GET

        tags: ["Public Like (Legacy)"]

        consumes: ["application/json", "application/octet-stream"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        isofficial: (isofficial) OPTIONAL bool in query

        limit: (limit) OPTIONAL int in query

        name: (name) OPTIONAL str in query

        offset: (offset) OPTIONAL int in query

        orderby: (orderby) OPTIONAL str in query

        sortby: (sortby) OPTIONAL str in query

        subtype: (subtype) OPTIONAL str in query

        tags: (tags) OPTIONAL List[str] in query

        type_: (type) OPTIONAL str in query

    Responses:
        200: OK - ModelsPaginatedContentDownloadResponse (Get liked contents)

        400: Bad Request - ResponseError (771100: unable to parse isofficial param)

        401: Unauthorized - ResponseError (20001: unauthorized access)

        500: Internal Server Error - ResponseError (771101: Unable to get ugc content: database error | 770801: Unable to get ugc content: database/Unable to get creator | 770803: Failed generate download URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetLikedContent.create(
        isofficial=isofficial,
        limit=limit,
        name=name,
        offset=offset,
        orderby=orderby,
        sortby=sortby,
        subtype=subtype,
        tags=tags,
        type_=type_,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetLikedContent)
async def get_liked_content_async(
    isofficial: Optional[bool] = None,
    limit: Optional[int] = None,
    name: Optional[str] = None,
    offset: Optional[int] = None,
    orderby: Optional[str] = None,
    sortby: Optional[str] = None,
    subtype: Optional[str] = None,
    tags: Optional[List[str]] = None,
    type_: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get liked contents (GetLikedContent)

    For advance tag filtering supports & as AND operator and | as OR operator and parentheses ( ) for priority. e.g:


    *tags=red*
    *tags=red&animal;*
    *tags=red|animal*
    *tags=red&animal;|wild*
    *tags=red&(animal|wild)*

    The precedence of logical operator is AND > OR, so if no parentheses, AND logical operator will be executed first.

    Allowed character for operand: alphanumeric, underscore _ and dash -

    Allowed character for operator: & | ( )


    **Please note that value of tags query param should be URL encoded**

    Properties:
        url: /ugc/v1/public/namespaces/{namespace}/contents/liked

        method: GET

        tags: ["Public Like (Legacy)"]

        consumes: ["application/json", "application/octet-stream"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        isofficial: (isofficial) OPTIONAL bool in query

        limit: (limit) OPTIONAL int in query

        name: (name) OPTIONAL str in query

        offset: (offset) OPTIONAL int in query

        orderby: (orderby) OPTIONAL str in query

        sortby: (sortby) OPTIONAL str in query

        subtype: (subtype) OPTIONAL str in query

        tags: (tags) OPTIONAL List[str] in query

        type_: (type) OPTIONAL str in query

    Responses:
        200: OK - ModelsPaginatedContentDownloadResponse (Get liked contents)

        400: Bad Request - ResponseError (771100: unable to parse isofficial param)

        401: Unauthorized - ResponseError (20001: unauthorized access)

        500: Internal Server Error - ResponseError (771101: Unable to get ugc content: database error | 770801: Unable to get ugc content: database/Unable to get creator | 770803: Failed generate download URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetLikedContent.create(
        isofficial=isofficial,
        limit=limit,
        name=name,
        offset=offset,
        orderby=orderby,
        sortby=sortby,
        subtype=subtype,
        tags=tags,
        type_=type_,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateContentLikeStatus)
def update_content_like_status(
    body: ModelsContentLikeRequest,
    content_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update like/unlike status to a content (UpdateContentLikeStatus)

    This endpoint will update like/unlike state from a content

    Properties:
        url: /ugc/v1/public/namespaces/{namespace}/contents/{contentId}/like

        method: PUT

        tags: ["Public Like (Legacy)"]

        consumes: ["application/json", "application/octet-stream"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsContentLikeRequest in body

        content_id: (contentId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsContentLikeResponse (Update like/unlike status to a content)

        400: Bad Request - ResponseError (771000: Malformed request/Content not found/Unable to update like status: content not found)

        401: Unauthorized - ResponseError (20001: unauthorized access)

        404: Not Found - ResponseError (771000: Malformed request/Content not found/Unable to update like status: content not found | 771001: unable to like content/Unable to update like status: database error | 771000: Malformed request/Content not found/Unable to update like status: content not found)

        500: Internal Server Error - ResponseError (771001: unable to like content/Unable to update like status: database error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateContentLikeStatus.create(
        body=body,
        content_id=content_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateContentLikeStatus)
async def update_content_like_status_async(
    body: ModelsContentLikeRequest,
    content_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update like/unlike status to a content (UpdateContentLikeStatus)

    This endpoint will update like/unlike state from a content

    Properties:
        url: /ugc/v1/public/namespaces/{namespace}/contents/{contentId}/like

        method: PUT

        tags: ["Public Like (Legacy)"]

        consumes: ["application/json", "application/octet-stream"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsContentLikeRequest in body

        content_id: (contentId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsContentLikeResponse (Update like/unlike status to a content)

        400: Bad Request - ResponseError (771000: Malformed request/Content not found/Unable to update like status: content not found)

        401: Unauthorized - ResponseError (20001: unauthorized access)

        404: Not Found - ResponseError (771000: Malformed request/Content not found/Unable to update like status: content not found | 771001: unable to like content/Unable to update like status: database error | 771000: Malformed request/Content not found/Unable to update like status: content not found)

        500: Internal Server Error - ResponseError (771001: unable to like content/Unable to update like status: database error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateContentLikeStatus.create(
        body=body,
        content_id=content_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



from datetime import datetime
from typing import Optional


def replace_resource(
    resource: str,
    namespace: Optional[str] = None,
    token_namespace: Optional[str] = None,
    publisher_namespace: Optional[str] = None,
    user_id: Optional[str] = None,
    cross_allowed: bool = False,
) -> str:
    new_resource = resource

    if (
        cross_allowed
        and token_namespace is not None
        and (publisher_namespace == token_namespace or publisher_namespace == namespace)
    ):
        new_resource = new_resource.replace("{namespace}", token_namespace)

    if namespace is not None:
        new_resource = new_resource.replace("{namespace}", namespace)

    if user_id is not None:
        new_resource = new_resource.replace("{userId}", user_id)

    return new_resource


def str2datetime(s: str) -> datetime:
    # from: 'YYYY-mm-ddTHH:MM:SS.fffffffffZ'
    # to:   'YYYY-mm-ddTHH:MM:SS.fffZ+0000'
    tz = "Z+0000" if s.endswith("Z") else ""  # Add explicit UTC timezone.
    s = s[0:23] + tz
    return datetime.strptime(s, "%Y-%m-%dT%H:%M:%S.%fZ%z")



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Gdpr Service."""

__version__ = "2.24.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .s2s_generate_personal_data_url import S2SGeneratePersonalDataURL
from .s2s_get_data_request_by_21f4a4 import S2SGetDataRequestByRequestID
from .s2s_get_list_finished_p_06385b import S2SGetListFinishedPersonalDataRequest
from .s2s_request_data_retrieval import S2SRequestDataRetrieval



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class ActionEnum(StrEnum):
    DECREMENT = "DECREMENT"
    DISABLE = "DISABLE"
    ENABLE = "ENABLE"
    GRANT = "GRANT"
    REVOKE = "REVOKE"
    SELL_BACK = "SELL_BACK"
    UPDATE = "UPDATE"


class ClazzEnum(StrEnum):
    APP = "APP"
    CODE = "CODE"
    ENTITLEMENT = "ENTITLEMENT"
    LOOTBOX = "LOOTBOX"
    MEDIA = "MEDIA"
    OPTIONBOX = "OPTIONBOX"
    SUBSCRIPTION = "SUBSCRIPTION"


class OriginEnum(StrEnum):
    EPIC = "Epic"
    GOOGLEPLAY = "GooglePlay"
    IOS = "IOS"
    NINTENDO = "Nintendo"
    OCULUS = "Oculus"
    OTHER = "Other"
    PLAYSTATION = "Playstation"
    STEAM = "Steam"
    SYSTEM = "System"
    TWITCH = "Twitch"
    XBOX = "Xbox"


class PublicEntitlementHistoryInfo(Model):
    """Public entitlement history info (PublicEntitlementHistoryInfo)

    Properties:
        action: (action) REQUIRED Union[str, ActionEnum]

        created_at: (createdAt) REQUIRED str

        entitlement_id: (entitlementId) REQUIRED str

        namespace: (namespace) REQUIRED str

        updated_at: (updatedAt) REQUIRED str

        user_id: (userId) REQUIRED str

        clazz: (clazz) OPTIONAL Union[str, ClazzEnum]

        item_id: (itemId) OPTIONAL str

        origin: (origin) OPTIONAL Union[str, OriginEnum]

        reason: (reason) OPTIONAL str

        sku: (sku) OPTIONAL str

        use_count: (useCount) OPTIONAL int

        use_count_change: (useCountChange) OPTIONAL int
    """

    # region fields

    action: Union[str, ActionEnum]  # REQUIRED
    created_at: str  # REQUIRED
    entitlement_id: str  # REQUIRED
    namespace: str  # REQUIRED
    updated_at: str  # REQUIRED
    user_id: str  # REQUIRED
    clazz: Union[str, ClazzEnum]  # OPTIONAL
    item_id: str  # OPTIONAL
    origin: Union[str, OriginEnum]  # OPTIONAL
    reason: str  # OPTIONAL
    sku: str  # OPTIONAL
    use_count: int  # OPTIONAL
    use_count_change: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_action(
        self, value: Union[str, ActionEnum]
    ) -> PublicEntitlementHistoryInfo:
        self.action = value
        return self

    def with_created_at(self, value: str) -> PublicEntitlementHistoryInfo:
        self.created_at = value
        return self

    def with_entitlement_id(self, value: str) -> PublicEntitlementHistoryInfo:
        self.entitlement_id = value
        return self

    def with_namespace(self, value: str) -> PublicEntitlementHistoryInfo:
        self.namespace = value
        return self

    def with_updated_at(self, value: str) -> PublicEntitlementHistoryInfo:
        self.updated_at = value
        return self

    def with_user_id(self, value: str) -> PublicEntitlementHistoryInfo:
        self.user_id = value
        return self

    def with_clazz(self, value: Union[str, ClazzEnum]) -> PublicEntitlementHistoryInfo:
        self.clazz = value
        return self

    def with_item_id(self, value: str) -> PublicEntitlementHistoryInfo:
        self.item_id = value
        return self

    def with_origin(
        self, value: Union[str, OriginEnum]
    ) -> PublicEntitlementHistoryInfo:
        self.origin = value
        return self

    def with_reason(self, value: str) -> PublicEntitlementHistoryInfo:
        self.reason = value
        return self

    def with_sku(self, value: str) -> PublicEntitlementHistoryInfo:
        self.sku = value
        return self

    def with_use_count(self, value: int) -> PublicEntitlementHistoryInfo:
        self.use_count = value
        return self

    def with_use_count_change(self, value: int) -> PublicEntitlementHistoryInfo:
        self.use_count_change = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "action"):
            result["action"] = str(self.action)
        elif include_empty:
            result["action"] = Union[str, ActionEnum]()
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "entitlement_id"):
            result["entitlementId"] = str(self.entitlement_id)
        elif include_empty:
            result["entitlementId"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "clazz"):
            result["clazz"] = str(self.clazz)
        elif include_empty:
            result["clazz"] = Union[str, ClazzEnum]()
        if hasattr(self, "item_id"):
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "origin"):
            result["origin"] = str(self.origin)
        elif include_empty:
            result["origin"] = Union[str, OriginEnum]()
        if hasattr(self, "reason"):
            result["reason"] = str(self.reason)
        elif include_empty:
            result["reason"] = ""
        if hasattr(self, "sku"):
            result["sku"] = str(self.sku)
        elif include_empty:
            result["sku"] = ""
        if hasattr(self, "use_count"):
            result["useCount"] = int(self.use_count)
        elif include_empty:
            result["useCount"] = 0
        if hasattr(self, "use_count_change"):
            result["useCountChange"] = int(self.use_count_change)
        elif include_empty:
            result["useCountChange"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        action: Union[str, ActionEnum],
        created_at: str,
        entitlement_id: str,
        namespace: str,
        updated_at: str,
        user_id: str,
        clazz: Optional[Union[str, ClazzEnum]] = None,
        item_id: Optional[str] = None,
        origin: Optional[Union[str, OriginEnum]] = None,
        reason: Optional[str] = None,
        sku: Optional[str] = None,
        use_count: Optional[int] = None,
        use_count_change: Optional[int] = None,
        **kwargs,
    ) -> PublicEntitlementHistoryInfo:
        instance = cls()
        instance.action = action
        instance.created_at = created_at
        instance.entitlement_id = entitlement_id
        instance.namespace = namespace
        instance.updated_at = updated_at
        instance.user_id = user_id
        if clazz is not None:
            instance.clazz = clazz
        if item_id is not None:
            instance.item_id = item_id
        if origin is not None:
            instance.origin = origin
        if reason is not None:
            instance.reason = reason
        if sku is not None:
            instance.sku = sku
        if use_count is not None:
            instance.use_count = use_count
        if use_count_change is not None:
            instance.use_count_change = use_count_change
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PublicEntitlementHistoryInfo:
        instance = cls()
        if not dict_:
            return instance
        if "action" in dict_ and dict_["action"] is not None:
            instance.action = str(dict_["action"])
        elif include_empty:
            instance.action = Union[str, ActionEnum]()
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "entitlementId" in dict_ and dict_["entitlementId"] is not None:
            instance.entitlement_id = str(dict_["entitlementId"])
        elif include_empty:
            instance.entitlement_id = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "clazz" in dict_ and dict_["clazz"] is not None:
            instance.clazz = str(dict_["clazz"])
        elif include_empty:
            instance.clazz = Union[str, ClazzEnum]()
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "origin" in dict_ and dict_["origin"] is not None:
            instance.origin = str(dict_["origin"])
        elif include_empty:
            instance.origin = Union[str, OriginEnum]()
        if "reason" in dict_ and dict_["reason"] is not None:
            instance.reason = str(dict_["reason"])
        elif include_empty:
            instance.reason = ""
        if "sku" in dict_ and dict_["sku"] is not None:
            instance.sku = str(dict_["sku"])
        elif include_empty:
            instance.sku = ""
        if "useCount" in dict_ and dict_["useCount"] is not None:
            instance.use_count = int(dict_["useCount"])
        elif include_empty:
            instance.use_count = 0
        if "useCountChange" in dict_ and dict_["useCountChange"] is not None:
            instance.use_count_change = int(dict_["useCountChange"])
        elif include_empty:
            instance.use_count_change = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, PublicEntitlementHistoryInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[PublicEntitlementHistoryInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        PublicEntitlementHistoryInfo,
        List[PublicEntitlementHistoryInfo],
        Dict[Any, PublicEntitlementHistoryInfo],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "action": "action",
            "createdAt": "created_at",
            "entitlementId": "entitlement_id",
            "namespace": "namespace",
            "updatedAt": "updated_at",
            "userId": "user_id",
            "clazz": "clazz",
            "itemId": "item_id",
            "origin": "origin",
            "reason": "reason",
            "sku": "sku",
            "useCount": "use_count",
            "useCountChange": "use_count_change",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "action": True,
            "createdAt": True,
            "entitlementId": True,
            "namespace": True,
            "updatedAt": True,
            "userId": True,
            "clazz": False,
            "itemId": False,
            "origin": False,
            "reason": False,
            "sku": False,
            "useCount": False,
            "useCountChange": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "action": [
                "DECREMENT",
                "DISABLE",
                "ENABLE",
                "GRANT",
                "REVOKE",
                "SELL_BACK",
                "UPDATE",
            ],
            "clazz": [
                "APP",
                "CODE",
                "ENTITLEMENT",
                "LOOTBOX",
                "MEDIA",
                "OPTIONBOX",
                "SUBSCRIPTION",
            ],
            "origin": [
                "Epic",
                "GooglePlay",
                "IOS",
                "Nintendo",
                "Oculus",
                "Other",
                "Playstation",
                "Steam",
                "System",
                "Twitch",
                "Xbox",
            ],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApimodelGetNotificationSubscriberStatusResponse(Model):
    """Apimodel get notification subscriber status response (apimodel.GetNotificationSubscriberStatusResponse)

    Properties:
        max_subscription: (maxSubscription) REQUIRED int

        notification_type: (notificationType) REQUIRED str

        subscribed: (subscribed) REQUIRED bool

        subscriptions_count: (subscriptionsCount) REQUIRED int

        display_name: (displayName) OPTIONAL str

        email_address: (emailAddress) OPTIONAL str

        user_id: (userId) OPTIONAL str
    """

    # region fields

    max_subscription: int  # REQUIRED
    notification_type: str  # REQUIRED
    subscribed: bool  # REQUIRED
    subscriptions_count: int  # REQUIRED
    display_name: str  # OPTIONAL
    email_address: str  # OPTIONAL
    user_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_max_subscription(
        self, value: int
    ) -> ApimodelGetNotificationSubscriberStatusResponse:
        self.max_subscription = value
        return self

    def with_notification_type(
        self, value: str
    ) -> ApimodelGetNotificationSubscriberStatusResponse:
        self.notification_type = value
        return self

    def with_subscribed(
        self, value: bool
    ) -> ApimodelGetNotificationSubscriberStatusResponse:
        self.subscribed = value
        return self

    def with_subscriptions_count(
        self, value: int
    ) -> ApimodelGetNotificationSubscriberStatusResponse:
        self.subscriptions_count = value
        return self

    def with_display_name(
        self, value: str
    ) -> ApimodelGetNotificationSubscriberStatusResponse:
        self.display_name = value
        return self

    def with_email_address(
        self, value: str
    ) -> ApimodelGetNotificationSubscriberStatusResponse:
        self.email_address = value
        return self

    def with_user_id(
        self, value: str
    ) -> ApimodelGetNotificationSubscriberStatusResponse:
        self.user_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "max_subscription"):
            result["maxSubscription"] = int(self.max_subscription)
        elif include_empty:
            result["maxSubscription"] = 0
        if hasattr(self, "notification_type"):
            result["notificationType"] = str(self.notification_type)
        elif include_empty:
            result["notificationType"] = ""
        if hasattr(self, "subscribed"):
            result["subscribed"] = bool(self.subscribed)
        elif include_empty:
            result["subscribed"] = False
        if hasattr(self, "subscriptions_count"):
            result["subscriptionsCount"] = int(self.subscriptions_count)
        elif include_empty:
            result["subscriptionsCount"] = 0
        if hasattr(self, "display_name"):
            result["displayName"] = str(self.display_name)
        elif include_empty:
            result["displayName"] = ""
        if hasattr(self, "email_address"):
            result["emailAddress"] = str(self.email_address)
        elif include_empty:
            result["emailAddress"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        max_subscription: int,
        notification_type: str,
        subscribed: bool,
        subscriptions_count: int,
        display_name: Optional[str] = None,
        email_address: Optional[str] = None,
        user_id: Optional[str] = None,
        **kwargs,
    ) -> ApimodelGetNotificationSubscriberStatusResponse:
        instance = cls()
        instance.max_subscription = max_subscription
        instance.notification_type = notification_type
        instance.subscribed = subscribed
        instance.subscriptions_count = subscriptions_count
        if display_name is not None:
            instance.display_name = display_name
        if email_address is not None:
            instance.email_address = email_address
        if user_id is not None:
            instance.user_id = user_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelGetNotificationSubscriberStatusResponse:
        instance = cls()
        if not dict_:
            return instance
        if "maxSubscription" in dict_ and dict_["maxSubscription"] is not None:
            instance.max_subscription = int(dict_["maxSubscription"])
        elif include_empty:
            instance.max_subscription = 0
        if "notificationType" in dict_ and dict_["notificationType"] is not None:
            instance.notification_type = str(dict_["notificationType"])
        elif include_empty:
            instance.notification_type = ""
        if "subscribed" in dict_ and dict_["subscribed"] is not None:
            instance.subscribed = bool(dict_["subscribed"])
        elif include_empty:
            instance.subscribed = False
        if "subscriptionsCount" in dict_ and dict_["subscriptionsCount"] is not None:
            instance.subscriptions_count = int(dict_["subscriptionsCount"])
        elif include_empty:
            instance.subscriptions_count = 0
        if "displayName" in dict_ and dict_["displayName"] is not None:
            instance.display_name = str(dict_["displayName"])
        elif include_empty:
            instance.display_name = ""
        if "emailAddress" in dict_ and dict_["emailAddress"] is not None:
            instance.email_address = str(dict_["emailAddress"])
        elif include_empty:
            instance.email_address = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelGetNotificationSubscriberStatusResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelGetNotificationSubscriberStatusResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelGetNotificationSubscriberStatusResponse,
        List[ApimodelGetNotificationSubscriberStatusResponse],
        Dict[Any, ApimodelGetNotificationSubscriberStatusResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "maxSubscription": "max_subscription",
            "notificationType": "notification_type",
            "subscribed": "subscribed",
            "subscriptionsCount": "subscriptions_count",
            "displayName": "display_name",
            "emailAddress": "email_address",
            "userId": "user_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "maxSubscription": True,
            "notificationType": True,
            "subscribed": True,
            "subscriptionsCount": True,
            "displayName": False,
            "emailAddress": False,
            "userId": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Challenge Service."""

__version__ = "1.30.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_claim_user_rewards import AdminClaimUserRewards
from .admin_claim_user_reward_b23bb0 import AdminClaimUserRewardsByGoalCode
from .admin_claim_users_rewards import AdminClaimUsersRewards
from .admin_get_user_rewards import AdminGetUserRewards
from .admin_get_user_rewards import (
    SortByEnum as AdminGetUserRewardsSortByEnum,
    StatusEnum as AdminGetUserRewardsStatusEnum,
)
from .public_claim_user_rewards import PublicClaimUserRewards
from .public_claim_user_rewar_049f52 import PublicClaimUserRewardsByGoalCode
from .public_get_user_rewards import PublicGetUserRewards
from .public_get_user_rewards import (
    SortByEnum as PublicGetUserRewardsSortByEnum,
    StatusEnum as PublicGetUserRewardsStatusEnum,
)



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Fleet Commander

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApiPagingInfo(Model):
    """Api paging info (api.PagingInfo)

    Properties:
        current_page: (currentPage) REQUIRED int

        has_next: (hasNext) REQUIRED bool

        has_pages: (hasPages) REQUIRED bool

        has_prev: (hasPrev) REQUIRED bool

        next_: (next) REQUIRED str

        page_nums: (pageNums) REQUIRED List[int]

        previous: (previous) REQUIRED str

        total: (total) REQUIRED int
    """

    # region fields

    current_page: int  # REQUIRED
    has_next: bool  # REQUIRED
    has_pages: bool  # REQUIRED
    has_prev: bool  # REQUIRED
    next_: str  # REQUIRED
    page_nums: List[int]  # REQUIRED
    previous: str  # REQUIRED
    total: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_current_page(self, value: int) -> ApiPagingInfo:
        self.current_page = value
        return self

    def with_has_next(self, value: bool) -> ApiPagingInfo:
        self.has_next = value
        return self

    def with_has_pages(self, value: bool) -> ApiPagingInfo:
        self.has_pages = value
        return self

    def with_has_prev(self, value: bool) -> ApiPagingInfo:
        self.has_prev = value
        return self

    def with_next(self, value: str) -> ApiPagingInfo:
        self.next_ = value
        return self

    def with_page_nums(self, value: List[int]) -> ApiPagingInfo:
        self.page_nums = value
        return self

    def with_previous(self, value: str) -> ApiPagingInfo:
        self.previous = value
        return self

    def with_total(self, value: int) -> ApiPagingInfo:
        self.total = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "current_page"):
            result["currentPage"] = int(self.current_page)
        elif include_empty:
            result["currentPage"] = 0
        if hasattr(self, "has_next"):
            result["hasNext"] = bool(self.has_next)
        elif include_empty:
            result["hasNext"] = False
        if hasattr(self, "has_pages"):
            result["hasPages"] = bool(self.has_pages)
        elif include_empty:
            result["hasPages"] = False
        if hasattr(self, "has_prev"):
            result["hasPrev"] = bool(self.has_prev)
        elif include_empty:
            result["hasPrev"] = False
        if hasattr(self, "next_"):
            result["next"] = str(self.next_)
        elif include_empty:
            result["next"] = ""
        if hasattr(self, "page_nums"):
            result["pageNums"] = [int(i0) for i0 in self.page_nums]
        elif include_empty:
            result["pageNums"] = []
        if hasattr(self, "previous"):
            result["previous"] = str(self.previous)
        elif include_empty:
            result["previous"] = ""
        if hasattr(self, "total"):
            result["total"] = int(self.total)
        elif include_empty:
            result["total"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        current_page: int,
        has_next: bool,
        has_pages: bool,
        has_prev: bool,
        next_: str,
        page_nums: List[int],
        previous: str,
        total: int,
        **kwargs,
    ) -> ApiPagingInfo:
        instance = cls()
        instance.current_page = current_page
        instance.has_next = has_next
        instance.has_pages = has_pages
        instance.has_prev = has_prev
        instance.next_ = next_
        instance.page_nums = page_nums
        instance.previous = previous
        instance.total = total
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApiPagingInfo:
        instance = cls()
        if not dict_:
            return instance
        if "currentPage" in dict_ and dict_["currentPage"] is not None:
            instance.current_page = int(dict_["currentPage"])
        elif include_empty:
            instance.current_page = 0
        if "hasNext" in dict_ and dict_["hasNext"] is not None:
            instance.has_next = bool(dict_["hasNext"])
        elif include_empty:
            instance.has_next = False
        if "hasPages" in dict_ and dict_["hasPages"] is not None:
            instance.has_pages = bool(dict_["hasPages"])
        elif include_empty:
            instance.has_pages = False
        if "hasPrev" in dict_ and dict_["hasPrev"] is not None:
            instance.has_prev = bool(dict_["hasPrev"])
        elif include_empty:
            instance.has_prev = False
        if "next" in dict_ and dict_["next"] is not None:
            instance.next_ = str(dict_["next"])
        elif include_empty:
            instance.next_ = ""
        if "pageNums" in dict_ and dict_["pageNums"] is not None:
            instance.page_nums = [int(i0) for i0 in dict_["pageNums"]]
        elif include_empty:
            instance.page_nums = []
        if "previous" in dict_ and dict_["previous"] is not None:
            instance.previous = str(dict_["previous"])
        elif include_empty:
            instance.previous = ""
        if "total" in dict_ and dict_["total"] is not None:
            instance.total = int(dict_["total"])
        elif include_empty:
            instance.total = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApiPagingInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApiPagingInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ApiPagingInfo, List[ApiPagingInfo], Dict[Any, ApiPagingInfo]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "currentPage": "current_page",
            "hasNext": "has_next",
            "hasPages": "has_pages",
            "hasPrev": "has_prev",
            "next": "next_",
            "pageNums": "page_nums",
            "previous": "previous",
            "total": "total",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "currentPage": True,
            "hasNext": True,
            "hasPages": True,
            "hasPrev": True,
            "next": True,
            "pageNums": True,
            "previous": True,
            "total": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.model_native_session_member import ModelNativeSessionMember


class ModelNativeSession(Model):
    """Model native session (model.NativeSession)

    Properties:
        created_at: (CreatedAt) REQUIRED str

        deleted_at: (DeletedAt) REQUIRED str

        id_: (id) REQUIRED str

        members: (members) REQUIRED List[ModelNativeSessionMember]

        namespace: (namespace) REQUIRED str

        native_session_platforms: (nativeSessionPlatforms) REQUIRED Dict[str, str]

        persistent: (persistent) REQUIRED bool

        push_context_id: (PushContextID) REQUIRED str
    """

    # region fields

    created_at: str  # REQUIRED
    deleted_at: str  # REQUIRED
    id_: str  # REQUIRED
    members: List[ModelNativeSessionMember]  # REQUIRED
    namespace: str  # REQUIRED
    native_session_platforms: Dict[str, str]  # REQUIRED
    persistent: bool  # REQUIRED
    push_context_id: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: str) -> ModelNativeSession:
        self.created_at = value
        return self

    def with_deleted_at(self, value: str) -> ModelNativeSession:
        self.deleted_at = value
        return self

    def with_id(self, value: str) -> ModelNativeSession:
        self.id_ = value
        return self

    def with_members(self, value: List[ModelNativeSessionMember]) -> ModelNativeSession:
        self.members = value
        return self

    def with_namespace(self, value: str) -> ModelNativeSession:
        self.namespace = value
        return self

    def with_native_session_platforms(
        self, value: Dict[str, str]
    ) -> ModelNativeSession:
        self.native_session_platforms = value
        return self

    def with_persistent(self, value: bool) -> ModelNativeSession:
        self.persistent = value
        return self

    def with_push_context_id(self, value: str) -> ModelNativeSession:
        self.push_context_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["CreatedAt"] = str(self.created_at)
        elif include_empty:
            result["CreatedAt"] = ""
        if hasattr(self, "deleted_at"):
            result["DeletedAt"] = str(self.deleted_at)
        elif include_empty:
            result["DeletedAt"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "members"):
            result["members"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.members
            ]
        elif include_empty:
            result["members"] = []
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "native_session_platforms"):
            result["nativeSessionPlatforms"] = {
                str(k0): str(v0) for k0, v0 in self.native_session_platforms.items()
            }
        elif include_empty:
            result["nativeSessionPlatforms"] = {}
        if hasattr(self, "persistent"):
            result["persistent"] = bool(self.persistent)
        elif include_empty:
            result["persistent"] = False
        if hasattr(self, "push_context_id"):
            result["PushContextID"] = str(self.push_context_id)
        elif include_empty:
            result["PushContextID"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: str,
        deleted_at: str,
        id_: str,
        members: List[ModelNativeSessionMember],
        namespace: str,
        native_session_platforms: Dict[str, str],
        persistent: bool,
        push_context_id: str,
        **kwargs,
    ) -> ModelNativeSession:
        instance = cls()
        instance.created_at = created_at
        instance.deleted_at = deleted_at
        instance.id_ = id_
        instance.members = members
        instance.namespace = namespace
        instance.native_session_platforms = native_session_platforms
        instance.persistent = persistent
        instance.push_context_id = push_context_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelNativeSession:
        instance = cls()
        if not dict_:
            return instance
        if "CreatedAt" in dict_ and dict_["CreatedAt"] is not None:
            instance.created_at = str(dict_["CreatedAt"])
        elif include_empty:
            instance.created_at = ""
        if "DeletedAt" in dict_ and dict_["DeletedAt"] is not None:
            instance.deleted_at = str(dict_["DeletedAt"])
        elif include_empty:
            instance.deleted_at = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "members" in dict_ and dict_["members"] is not None:
            instance.members = [
                ModelNativeSessionMember.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["members"]
            ]
        elif include_empty:
            instance.members = []
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if (
            "nativeSessionPlatforms" in dict_
            and dict_["nativeSessionPlatforms"] is not None
        ):
            instance.native_session_platforms = {
                str(k0): str(v0) for k0, v0 in dict_["nativeSessionPlatforms"].items()
            }
        elif include_empty:
            instance.native_session_platforms = {}
        if "persistent" in dict_ and dict_["persistent"] is not None:
            instance.persistent = bool(dict_["persistent"])
        elif include_empty:
            instance.persistent = False
        if "PushContextID" in dict_ and dict_["PushContextID"] is not None:
            instance.push_context_id = str(dict_["PushContextID"])
        elif include_empty:
            instance.push_context_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelNativeSession]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelNativeSession]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelNativeSession, List[ModelNativeSession], Dict[Any, ModelNativeSession]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "CreatedAt": "created_at",
            "DeletedAt": "deleted_at",
            "id": "id_",
            "members": "members",
            "namespace": "namespace",
            "nativeSessionPlatforms": "native_session_platforms",
            "persistent": "persistent",
            "PushContextID": "push_context_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "CreatedAt": True,
            "DeletedAt": True,
            "id": True,
            "members": True,
            "namespace": True,
            "nativeSessionPlatforms": True,
            "persistent": True,
            "PushContextID": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Inventory Service Early Access

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApimodelsItemResp(Model):
    """Apimodels item resp (apimodels.ItemResp)

    Properties:
        created_at: (createdAt) REQUIRED str

        custom_attributes: (customAttributes) REQUIRED Dict[str, Any]

        id_: (id) REQUIRED str

        inventory_id: (inventoryId) REQUIRED str

        item_info: (itemInfo) REQUIRED Dict[str, Any]

        namespace: (namespace) REQUIRED str

        qty: (qty) REQUIRED int

        server_custom_attributes: (serverCustomAttributes) REQUIRED Dict[str, Any]

        slot_id: (slotId) REQUIRED str

        slot_used: (slotUsed) REQUIRED int

        source: (source) REQUIRED str

        source_item_id: (sourceItemId) REQUIRED str

        tags: (tags) REQUIRED List[str]

        type_: (type) REQUIRED str

        updated_at: (updatedAt) REQUIRED str

        user_id: (userId) REQUIRED str

        platform_available: (platformAvailable) OPTIONAL bool
    """

    # region fields

    created_at: str  # REQUIRED
    custom_attributes: Dict[str, Any]  # REQUIRED
    id_: str  # REQUIRED
    inventory_id: str  # REQUIRED
    item_info: Dict[str, Any]  # REQUIRED
    namespace: str  # REQUIRED
    qty: int  # REQUIRED
    server_custom_attributes: Dict[str, Any]  # REQUIRED
    slot_id: str  # REQUIRED
    slot_used: int  # REQUIRED
    source: str  # REQUIRED
    source_item_id: str  # REQUIRED
    tags: List[str]  # REQUIRED
    type_: str  # REQUIRED
    updated_at: str  # REQUIRED
    user_id: str  # REQUIRED
    platform_available: bool  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: str) -> ApimodelsItemResp:
        self.created_at = value
        return self

    def with_custom_attributes(self, value: Dict[str, Any]) -> ApimodelsItemResp:
        self.custom_attributes = value
        return self

    def with_id(self, value: str) -> ApimodelsItemResp:
        self.id_ = value
        return self

    def with_inventory_id(self, value: str) -> ApimodelsItemResp:
        self.inventory_id = value
        return self

    def with_item_info(self, value: Dict[str, Any]) -> ApimodelsItemResp:
        self.item_info = value
        return self

    def with_namespace(self, value: str) -> ApimodelsItemResp:
        self.namespace = value
        return self

    def with_qty(self, value: int) -> ApimodelsItemResp:
        self.qty = value
        return self

    def with_server_custom_attributes(self, value: Dict[str, Any]) -> ApimodelsItemResp:
        self.server_custom_attributes = value
        return self

    def with_slot_id(self, value: str) -> ApimodelsItemResp:
        self.slot_id = value
        return self

    def with_slot_used(self, value: int) -> ApimodelsItemResp:
        self.slot_used = value
        return self

    def with_source(self, value: str) -> ApimodelsItemResp:
        self.source = value
        return self

    def with_source_item_id(self, value: str) -> ApimodelsItemResp:
        self.source_item_id = value
        return self

    def with_tags(self, value: List[str]) -> ApimodelsItemResp:
        self.tags = value
        return self

    def with_type(self, value: str) -> ApimodelsItemResp:
        self.type_ = value
        return self

    def with_updated_at(self, value: str) -> ApimodelsItemResp:
        self.updated_at = value
        return self

    def with_user_id(self, value: str) -> ApimodelsItemResp:
        self.user_id = value
        return self

    def with_platform_available(self, value: bool) -> ApimodelsItemResp:
        self.platform_available = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "custom_attributes"):
            result["customAttributes"] = {
                str(k0): v0 for k0, v0 in self.custom_attributes.items()
            }
        elif include_empty:
            result["customAttributes"] = {}
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "inventory_id"):
            result["inventoryId"] = str(self.inventory_id)
        elif include_empty:
            result["inventoryId"] = ""
        if hasattr(self, "item_info"):
            result["itemInfo"] = {str(k0): v0 for k0, v0 in self.item_info.items()}
        elif include_empty:
            result["itemInfo"] = {}
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "qty"):
            result["qty"] = int(self.qty)
        elif include_empty:
            result["qty"] = 0
        if hasattr(self, "server_custom_attributes"):
            result["serverCustomAttributes"] = {
                str(k0): v0 for k0, v0 in self.server_custom_attributes.items()
            }
        elif include_empty:
            result["serverCustomAttributes"] = {}
        if hasattr(self, "slot_id"):
            result["slotId"] = str(self.slot_id)
        elif include_empty:
            result["slotId"] = ""
        if hasattr(self, "slot_used"):
            result["slotUsed"] = int(self.slot_used)
        elif include_empty:
            result["slotUsed"] = 0
        if hasattr(self, "source"):
            result["source"] = str(self.source)
        elif include_empty:
            result["source"] = ""
        if hasattr(self, "source_item_id"):
            result["sourceItemId"] = str(self.source_item_id)
        elif include_empty:
            result["sourceItemId"] = ""
        if hasattr(self, "tags"):
            result["tags"] = [str(i0) for i0 in self.tags]
        elif include_empty:
            result["tags"] = []
        if hasattr(self, "type_"):
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "platform_available"):
            result["platformAvailable"] = bool(self.platform_available)
        elif include_empty:
            result["platformAvailable"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: str,
        custom_attributes: Dict[str, Any],
        id_: str,
        inventory_id: str,
        item_info: Dict[str, Any],
        namespace: str,
        qty: int,
        server_custom_attributes: Dict[str, Any],
        slot_id: str,
        slot_used: int,
        source: str,
        source_item_id: str,
        tags: List[str],
        type_: str,
        updated_at: str,
        user_id: str,
        platform_available: Optional[bool] = None,
        **kwargs,
    ) -> ApimodelsItemResp:
        instance = cls()
        instance.created_at = created_at
        instance.custom_attributes = custom_attributes
        instance.id_ = id_
        instance.inventory_id = inventory_id
        instance.item_info = item_info
        instance.namespace = namespace
        instance.qty = qty
        instance.server_custom_attributes = server_custom_attributes
        instance.slot_id = slot_id
        instance.slot_used = slot_used
        instance.source = source
        instance.source_item_id = source_item_id
        instance.tags = tags
        instance.type_ = type_
        instance.updated_at = updated_at
        instance.user_id = user_id
        if platform_available is not None:
            instance.platform_available = platform_available
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsItemResp:
        instance = cls()
        if not dict_:
            return instance
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "customAttributes" in dict_ and dict_["customAttributes"] is not None:
            instance.custom_attributes = {
                str(k0): v0 for k0, v0 in dict_["customAttributes"].items()
            }
        elif include_empty:
            instance.custom_attributes = {}
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "inventoryId" in dict_ and dict_["inventoryId"] is not None:
            instance.inventory_id = str(dict_["inventoryId"])
        elif include_empty:
            instance.inventory_id = ""
        if "itemInfo" in dict_ and dict_["itemInfo"] is not None:
            instance.item_info = {str(k0): v0 for k0, v0 in dict_["itemInfo"].items()}
        elif include_empty:
            instance.item_info = {}
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "qty" in dict_ and dict_["qty"] is not None:
            instance.qty = int(dict_["qty"])
        elif include_empty:
            instance.qty = 0
        if (
            "serverCustomAttributes" in dict_
            and dict_["serverCustomAttributes"] is not None
        ):
            instance.server_custom_attributes = {
                str(k0): v0 for k0, v0 in dict_["serverCustomAttributes"].items()
            }
        elif include_empty:
            instance.server_custom_attributes = {}
        if "slotId" in dict_ and dict_["slotId"] is not None:
            instance.slot_id = str(dict_["slotId"])
        elif include_empty:
            instance.slot_id = ""
        if "slotUsed" in dict_ and dict_["slotUsed"] is not None:
            instance.slot_used = int(dict_["slotUsed"])
        elif include_empty:
            instance.slot_used = 0
        if "source" in dict_ and dict_["source"] is not None:
            instance.source = str(dict_["source"])
        elif include_empty:
            instance.source = ""
        if "sourceItemId" in dict_ and dict_["sourceItemId"] is not None:
            instance.source_item_id = str(dict_["sourceItemId"])
        elif include_empty:
            instance.source_item_id = ""
        if "tags" in dict_ and dict_["tags"] is not None:
            instance.tags = [str(i0) for i0 in dict_["tags"]]
        elif include_empty:
            instance.tags = []
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "platformAvailable" in dict_ and dict_["platformAvailable"] is not None:
            instance.platform_available = bool(dict_["platformAvailable"])
        elif include_empty:
            instance.platform_available = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsItemResp]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsItemResp]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsItemResp, List[ApimodelsItemResp], Dict[Any, ApimodelsItemResp]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdAt": "created_at",
            "customAttributes": "custom_attributes",
            "id": "id_",
            "inventoryId": "inventory_id",
            "itemInfo": "item_info",
            "namespace": "namespace",
            "qty": "qty",
            "serverCustomAttributes": "server_custom_attributes",
            "slotId": "slot_id",
            "slotUsed": "slot_used",
            "source": "source",
            "sourceItemId": "source_item_id",
            "tags": "tags",
            "type": "type_",
            "updatedAt": "updated_at",
            "userId": "user_id",
            "platformAvailable": "platform_available",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdAt": True,
            "customAttributes": True,
            "id": True,
            "inventoryId": True,
            "itemInfo": True,
            "namespace": True,
            "qty": True,
            "serverCustomAttributes": True,
            "slotId": True,
            "slotUsed": True,
            "source": True,
            "sourceItemId": True,
            "tags": True,
            "type": True,
            "updatedAt": True,
            "userId": True,
            "platformAvailable": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import AccountcommonTagResponse
from ..models import AccountcommonTagsGetResponseV3
from ..models import ModelTagCreateRequestV3
from ..models import ModelTagUpdateRequestV3
from ..models import RestErrorResponse

from ..operations.account_idenfifier_tag import AdminCreateTagV3
from ..operations.account_idenfifier_tag import AdminDeleteTagV3
from ..operations.account_idenfifier_tag import AdminQueryTagV3
from ..operations.account_idenfifier_tag import AdminUpdateTagV3


@same_doc_as(AdminCreateTagV3)
def admin_create_tag_v3(
    body: ModelTagCreateRequestV3,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Account Identifier Tag (AdminCreateTagV3)

    Create a new Account Identifier Tag for users. This endpoint allows administrators to create tags that can be used to identify and categorize user accounts.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags

        method: POST

        tags: ["Account Idenfifier Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelTagCreateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - AccountcommonTagResponse (Created)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error)

        409: Conflict - RestErrorResponse

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminCreateTagV3.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminCreateTagV3)
async def admin_create_tag_v3_async(
    body: ModelTagCreateRequestV3,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Account Identifier Tag (AdminCreateTagV3)

    Create a new Account Identifier Tag for users. This endpoint allows administrators to create tags that can be used to identify and categorize user accounts.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags

        method: POST

        tags: ["Account Idenfifier Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelTagCreateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - AccountcommonTagResponse (Created)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error)

        409: Conflict - RestErrorResponse

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminCreateTagV3.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminDeleteTagV3)
def admin_delete_tag_v3(
    tag_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Account Identifier Tag (AdminDeleteTagV3)

    Delete an Account Identifier Tag. This endpoint allows administrators to delete a tag that is used to identify and categorize user accounts.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags/{tagId}

        method: DELETE

        tags: ["Account Idenfifier Tag"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        tag_id: (tagId) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        404: Not Found - RestErrorResponse

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteTagV3.create(
        tag_id=tag_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeleteTagV3)
async def admin_delete_tag_v3_async(
    tag_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Account Identifier Tag (AdminDeleteTagV3)

    Delete an Account Identifier Tag. This endpoint allows administrators to delete a tag that is used to identify and categorize user accounts.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags/{tagId}

        method: DELETE

        tags: ["Account Idenfifier Tag"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        tag_id: (tagId) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        404: Not Found - RestErrorResponse

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteTagV3.create(
        tag_id=tag_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminQueryTagV3)
def admin_query_tag_v3(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    tag_name: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query Account Identifier Tag (AdminQueryTagV3)

    Retrieve Account Identifier Tags. This endpoint allows administrators to retrieve tags that are used to identify and categorize user accounts.
    Tag Name can be used for partial content search.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags

        method: GET

        tags: ["Account Idenfifier Tag"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        tag_name: (tagName) OPTIONAL str in query

    Responses:
        200: OK - AccountcommonTagsGetResponseV3 (OK)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminQueryTagV3.create(
        limit=limit,
        offset=offset,
        tag_name=tag_name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminQueryTagV3)
async def admin_query_tag_v3_async(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    tag_name: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query Account Identifier Tag (AdminQueryTagV3)

    Retrieve Account Identifier Tags. This endpoint allows administrators to retrieve tags that are used to identify and categorize user accounts.
    Tag Name can be used for partial content search.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags

        method: GET

        tags: ["Account Idenfifier Tag"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        tag_name: (tagName) OPTIONAL str in query

    Responses:
        200: OK - AccountcommonTagsGetResponseV3 (OK)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminQueryTagV3.create(
        limit=limit,
        offset=offset,
        tag_name=tag_name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateTagV3)
def admin_update_tag_v3(
    body: ModelTagUpdateRequestV3,
    tag_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Account Identifier Tag (AdminUpdateTagV3)

    Update an existing Account Identifier Tag. This endpoint allows administrators to update the details of a tag that is used to identify and categorize user accounts.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags/{tagId}

        method: PUT

        tags: ["Account Idenfifier Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelTagUpdateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

        tag_id: (tagId) REQUIRED str in path

    Responses:
        200: OK - AccountcommonTagResponse (OK)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error)

        404: Not Found - RestErrorResponse

        409: Conflict - RestErrorResponse

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateTagV3.create(
        body=body,
        tag_id=tag_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateTagV3)
async def admin_update_tag_v3_async(
    body: ModelTagUpdateRequestV3,
    tag_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Account Identifier Tag (AdminUpdateTagV3)

    Update an existing Account Identifier Tag. This endpoint allows administrators to update the details of a tag that is used to identify and categorize user accounts.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags/{tagId}

        method: PUT

        tags: ["Account Idenfifier Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelTagUpdateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

        tag_id: (tagId) REQUIRED str in path

    Responses:
        200: OK - AccountcommonTagResponse (OK)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error)

        404: Not Found - RestErrorResponse

        409: Conflict - RestErrorResponse

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateTagV3.create(
        body=body,
        tag_id=tag_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Ugc Service."""

__version__ = "2.28.2"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_approve_staging_content import AdminApproveStagingContent
from .admin_get_staging_conte_7fe68e import AdminGetStagingContentByID
from .admin_list_staging_contents import AdminListStagingContents
from .admin_list_user_staging_d2fe4e import AdminListUserStagingContents



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the Custom Service Manager."""

__version__ = "1.34.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .delete_secret_v1 import DeleteSecretV1
from .delete_variable_v1 import DeleteVariableV1
from .get_list_of_secrets_v1 import GetListOfSecretsV1
from .get_list_of_variables_v1 import GetListOfVariablesV1
from .save_secret_v1 import SaveSecretV1
from .save_variable_v1 import SaveVariableV1
from .update_secret_v1 import UpdateSecretV1
from .update_variable_v1 import UpdateVariableV1



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Achievement Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelsUserContributionResponse(Model):
    """Models user contribution response (models.UserContributionResponse)

    Properties:
        achievement_code: (achievementCode) REQUIRED str

        can_claim_reward: (canClaimReward) REQUIRED bool

        contributed_value: (contributedValue) REQUIRED float

        id_: (id) REQUIRED str

        name: (name) REQUIRED Dict[str, str]

        namespace: (namespace) REQUIRED str

        user_id: (userId) REQUIRED str
    """

    # region fields

    achievement_code: str  # REQUIRED
    can_claim_reward: bool  # REQUIRED
    contributed_value: float  # REQUIRED
    id_: str  # REQUIRED
    name: Dict[str, str]  # REQUIRED
    namespace: str  # REQUIRED
    user_id: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_achievement_code(self, value: str) -> ModelsUserContributionResponse:
        self.achievement_code = value
        return self

    def with_can_claim_reward(self, value: bool) -> ModelsUserContributionResponse:
        self.can_claim_reward = value
        return self

    def with_contributed_value(self, value: float) -> ModelsUserContributionResponse:
        self.contributed_value = value
        return self

    def with_id(self, value: str) -> ModelsUserContributionResponse:
        self.id_ = value
        return self

    def with_name(self, value: Dict[str, str]) -> ModelsUserContributionResponse:
        self.name = value
        return self

    def with_namespace(self, value: str) -> ModelsUserContributionResponse:
        self.namespace = value
        return self

    def with_user_id(self, value: str) -> ModelsUserContributionResponse:
        self.user_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "achievement_code"):
            result["achievementCode"] = str(self.achievement_code)
        elif include_empty:
            result["achievementCode"] = ""
        if hasattr(self, "can_claim_reward"):
            result["canClaimReward"] = bool(self.can_claim_reward)
        elif include_empty:
            result["canClaimReward"] = False
        if hasattr(self, "contributed_value"):
            result["contributedValue"] = float(self.contributed_value)
        elif include_empty:
            result["contributedValue"] = 0.0
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "name"):
            result["name"] = {str(k0): str(v0) for k0, v0 in self.name.items()}
        elif include_empty:
            result["name"] = {}
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        achievement_code: str,
        can_claim_reward: bool,
        contributed_value: float,
        id_: str,
        name: Dict[str, str],
        namespace: str,
        user_id: str,
        **kwargs,
    ) -> ModelsUserContributionResponse:
        instance = cls()
        instance.achievement_code = achievement_code
        instance.can_claim_reward = can_claim_reward
        instance.contributed_value = contributed_value
        instance.id_ = id_
        instance.name = name
        instance.namespace = namespace
        instance.user_id = user_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsUserContributionResponse:
        instance = cls()
        if not dict_:
            return instance
        if "achievementCode" in dict_ and dict_["achievementCode"] is not None:
            instance.achievement_code = str(dict_["achievementCode"])
        elif include_empty:
            instance.achievement_code = ""
        if "canClaimReward" in dict_ and dict_["canClaimReward"] is not None:
            instance.can_claim_reward = bool(dict_["canClaimReward"])
        elif include_empty:
            instance.can_claim_reward = False
        if "contributedValue" in dict_ and dict_["contributedValue"] is not None:
            instance.contributed_value = float(dict_["contributedValue"])
        elif include_empty:
            instance.contributed_value = 0.0
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = {str(k0): str(v0) for k0, v0 in dict_["name"].items()}
        elif include_empty:
            instance.name = {}
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsUserContributionResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsUserContributionResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsUserContributionResponse,
        List[ModelsUserContributionResponse],
        Dict[Any, ModelsUserContributionResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "achievementCode": "achievement_code",
            "canClaimReward": "can_claim_reward",
            "contributedValue": "contributed_value",
            "id": "id_",
            "name": "name",
            "namespace": "namespace",
            "userId": "user_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "achievementCode": True,
            "canClaimReward": True,
            "contributedValue": True,
            "id": True,
            "name": True,
            "namespace": True,
            "userId": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Platform Service."""

__version__ = "6.14.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .create_payment_provider_config import CreatePaymentProviderConfig
from .debug_matched_payment_m_9bf142 import DebugMatchedPaymentMerchantConfig
from .debug_matched_payment_p_02dcf1 import DebugMatchedPaymentProviderConfig
from .delete_payment_provider_config import DeletePaymentProviderConfig
from .get_aggregate_payment_p_41c63c import GetAggregatePaymentProviders
from .get_payment_merchant_config import GetPaymentMerchantConfig
from .get_payment_merchant_config_1 import GetPaymentMerchantConfig1
from .get_payment_tax_config import GetPaymentTaxConfig
from .get_special_payment_providers import GetSpecialPaymentProviders
from .query_payment_provider_config import QueryPaymentProviderConfig
from .test_adyen_config import TestAdyenConfig
from .test_adyen_config_by_id import TestAdyenConfigById
from .test_ali_pay_config import TestAliPayConfig
from .test_ali_pay_config_by_id import TestAliPayConfigById
from .test_checkout_config import TestCheckoutConfig
from .test_checkout_config_by_id import TestCheckoutConfigById
from .test_neon_pay_config import TestNeonPayConfig
from .test_neon_pay_config_by_id import TestNeonPayConfigById
from .test_pay_pal_config import TestPayPalConfig
from .test_pay_pal_config_by_id import TestPayPalConfigById
from .test_stripe_config import TestStripeConfig
from .test_stripe_config_by_id import TestStripeConfigById
from .test_wx_pay_config import TestWxPayConfig
from .test_wx_pay_config_by_id import TestWxPayConfigById
from .test_xsolla_config import TestXsollaConfig
from .test_xsolla_config_by_id import TestXsollaConfigById
from .update_adyen_config import UpdateAdyenConfig
from .update_ali_pay_config import UpdateAliPayConfig
from .update_checkout_config import UpdateCheckoutConfig
from .update_neon_pay_config import UpdateNeonPayConfig
from .update_pay_pal_config import UpdatePayPalConfig
from .update_payment_domain_w_db8e8d import UpdatePaymentDomainWhitelistConfig
from .update_payment_provider_config import UpdatePaymentProviderConfig
from .update_payment_tax_config import UpdatePaymentTaxConfig
from .update_stripe_config import UpdateStripeConfig
from .update_wx_pay_config import UpdateWxPayConfig
from .update_wx_pay_config_cert import UpdateWxPayConfigCert
from .update_xsolla_config import UpdateXsollaConfig
from .update_xsolla_ui_config import UpdateXsollaUIConfig



import logging
import os
from threading import Event, RLock
from typing import Any, Dict, List, Optional, Tuple, Union

import jwt

import accelbyte_py_sdk.api.basic as basic_service
import accelbyte_py_sdk.api.basic.models as basic_models
import accelbyte_py_sdk.api.iam as iam_service
import accelbyte_py_sdk.api.iam.models as iam_models
from accelbyte_py_sdk.core import AccelByteSDK, Timer

from ._bloom_filter import BloomFilter
from ._ctypes import (
    FetchNamespaceContextError,
    FetchRoleError,
    PermissionStruct,
    create_permission_struct,
)
from ._utils import replace_resource, str2datetime

logger = logging.getLogger(__name__)

PublicPrivateKey = Any
JWTClaims = Dict[str, Any]
JWKSet = Dict[str, PublicPrivateKey]
NamespaceContext = basic_models.NamespaceContext
NamespaceRole = Dict[str, str]
Role = iam_models.ModelRolePermissionResponseV3


class _InflightFetch:
    """Single-flight coordination record: one fetcher per key; others wait on the event.

    `error` holds either a service-layer error object (returned from the
    (result, error) tuple) or a raised BaseException. Waiters distinguish the
    two and re-raise exceptions to preserve the original traceback.
    """

    __slots__ = ("event", "error")

    def __init__(self) -> None:
        self.event: Event = Event()
        # None = no error; BaseException subtype = raised exception captured by
        # the fetcher; otherwise a service-layer error tuple value. Typed as
        # Optional[Any] because Union[BaseException, Any] collapses to Any.
        self.error: Optional[Any] = None


class JWKSCache(Timer):
    JWKS_KEYS_KEY: str = "keys"

    def __init__(
        self,
        sdk: AccelByteSDK,
        interval: Optional[Union[int, float]] = None,
        raise_on_error: bool = False,
        **kwargs,
    ):
        self.sdk = sdk
        self._jwks: Dict[str, PublicPrivateKey] = {}
        self._lock = RLock()
        self._raise_on_error = raise_on_error
        if interval is not None:
            Timer.__init__(
                self,
                interval,
                self.update,
                daemon=True,
                repeats=-1,
                autostart=True,
                repeat_on_exception=True,
            )

    def update(self, **kwargs):
        result, error = iam_service.get_jwksv3(
            x_additional_headers=kwargs.get("x_additional_headers", None),
            sdk=self.sdk,
        )
        if error:
            if self._raise_on_error:
                raise Exception(error)
            return

        with self._lock:
            keys = result.to_dict().get(JWKSCache.JWKS_KEYS_KEY, [])
            jwks = jwt.PyJWKSet(keys)
            for jwk in jwks.keys:
                self._jwks[jwk.key_id] = jwk.key

    def get_key_from_cache(self, key_id: str, **kwargs) -> Optional[PublicPrivateKey]:
        with self._lock:
            return self._jwks.get(key_id, None)

    def get_key(self, key_id: str, **kwargs) -> Optional[PublicPrivateKey]:
        key = self.get_key_from_cache(key_id)
        if key is not None:
            return key

        self.update(**kwargs)

        return self.get_key_from_cache(key_id)


class NamespaceContextCache(Timer):
    ENV_VAR_FALLBACK: str = "AB_NAMESPACE_CONTEXT_FALLBACK"

    def __init__(
        self,
        sdk: AccelByteSDK,
        interval: Optional[Union[int, float]] = None,
        raise_on_error: bool = True,
        namespace_context_fallback: Optional[bool] = None,
        **kwargs,
    ):
        self.sdk = sdk
        self._namespace_contexts: Dict[str, Any] = {}
        self._inflight: Dict[str, _InflightFetch] = {}
        self._lock = RLock()
        self._raise_on_error = raise_on_error
        self._generation: int = 0

        if namespace_context_fallback is not None:
            self._namespace_context_fallback = namespace_context_fallback
        else:
            env_val = (
                os.environ.get(NamespaceContextCache.ENV_VAR_FALLBACK, "")
                .strip()
                .lower()
            )
            self._namespace_context_fallback = (
                env_val not in ("0", "false", "no", "off", "disabled")
                if env_val
                else True
            )

        if interval is not None:
            Timer.__init__(
                self,
                interval,
                self.refresh,
                daemon=True,
                repeats=-1,
                autostart=True,
                repeat_on_exception=True,
            )

    def clear(self):
        """Drop all cached entries and pending single-flight coordinators.

        Bumps the generation counter so any concurrent refresh() aborts before
        re-populating the cleared cache. Signals waiters on outstanding
        coordinators before discarding them so no thread blocks indefinitely.
        Waiters woken this way observe inflight.error == None (the fetch had
        not yet completed) and surface a transient None / 401 to callers --
        the expected semantic for a deliberately invalidated cache.

        The fetcher's finally block may call event.set() again on a coordinator
        already signalled here; threading.Event.set() is idempotent.
        """
        with self._lock:
            self._generation += 1
            for fetch in self._inflight.values():
                fetch.event.set()
            self._namespace_contexts = {}
            self._inflight = {}

    def update(self, **kwargs):
        pass

    def refresh(self, **kwargs):
        """Re-fetch every currently-cached namespace context in place.

        On per-key failure the prior entry is retained (serve-stale), so a transient
        IAM outage cannot empty the cache and cascade into permission denials.

        Invoked by the background Timer with no kwargs, so x_additional_headers is
        not propagated during automatic refreshes; auth flows through sdk=self.sdk.
        Live callers entering get_namespace_context for a refreshed key hit the
        cached entry before the single-flight gate, so they never block on the
        in-progress refresh. Aborts immediately if clear() bumps the generation
        counter mid-loop, so external invalidation is not silently undone.
        Keys currently in-flight (in _inflight but not yet in _namespace_contexts)
        are not included in the snapshot; they will be picked up next cycle.
        """
        with self._lock:
            gen = self._generation
            namespaces = list(self._namespace_contexts.keys())
        for namespace in namespaces:
            with self._lock:
                if self._generation != gen:
                    return
            try:
                error = self.cache_namespace_context(namespace=namespace, **kwargs)
            except Exception as exc:
                logger.debug(
                    "namespace context refresh raised for '%s': %s; keeping stale entry",
                    namespace,
                    exc,
                )
                continue
            if error:
                logger.debug(
                    "namespace context refresh failed for '%s': %s; keeping stale entry",
                    namespace,
                    error,
                )
                continue
            # If clear() ran during the fetch, undo the spurious write so the
            # invalidation is not silently defeated by this refresh cycle.
            with self._lock:
                if self._generation != gen:
                    self._namespace_contexts.pop(namespace, None)
                    return

    def cache_namespace_context(self, namespace: str, **kwargs) -> Any:
        with self._lock:
            inflight = self._inflight.get(namespace)
            if inflight is None:
                inflight = _InflightFetch()
                self._inflight[namespace] = inflight
                is_fetcher = True
            else:
                is_fetcher = False

        if not is_fetcher:
            logger.debug(
                "cache waiter: namespace=%s, waiting for in-flight fetch", namespace
            )
            inflight.event.wait()
            err = inflight.error
            logger.debug(
                "cache waiter: namespace=%s, unblocked, error=%r", namespace, err
            )
            if isinstance(err, BaseException):
                raise err
            return err

        try:
            try:
                namespace_context, error = basic_service.get_namespace_context(
                    namespace=namespace,
                    x_additional_headers=kwargs.get("x_additional_headers", None),
                    sdk=self.sdk,
                )
            except BaseException as exc:
                inflight.error = exc
                raise
            if error:
                inflight.error = error
                return error
            with self._lock:
                self._namespace_contexts[namespace] = namespace_context
            return None
        finally:
            with self._lock:
                self._inflight.pop(namespace, None)
                inflight.event.set()

    def get_namespace_context(
        self, namespace: str, **kwargs
    ) -> Optional[NamespaceContext]:
        with self._lock:
            namespace_context = self._namespace_contexts.get(namespace, None)
            if namespace_context:
                return namespace_context

        # Lock released; concurrent callers for the same namespace are coalesced
        # inside cache_namespace_context via _InflightFetch. Do not remove that
        # coordination without also re-acquiring the lock around the fetch.
        error = self.cache_namespace_context(namespace=namespace, **kwargs)
        if error:
            if self._namespace_context_fallback:
                logger.debug(
                    "namespace context fetch failed for '%s': %s; attempting fallback",
                    namespace,
                    error,
                )
                derived = self._derive_namespace_context_from_app_ns(
                    namespace=namespace, **kwargs
                )
                if derived is not None:
                    return derived
            if self._raise_on_error:
                raise FetchNamespaceContextError(f"namespace context: {namespace}")
            return None

        with self._lock:
            return self._namespace_contexts.get(namespace, None)

    def _derive_namespace_context_from_app_ns(
        self, namespace: str, **kwargs
    ) -> Optional[NamespaceContext]:
        """Fallback for when the app lacks permission to query a parent namespace.

        Fetches the app's own game namespace context (from the SDK config) and derives
        studio/publisher namespace contexts from it, populating the cache so that
        permission validation can proceed without a 403.
        """
        app_namespace, error = self.sdk.get_namespace()
        if error or not app_namespace or app_namespace == namespace:
            return None

        with self._lock:
            app_context = self._namespace_contexts.get(app_namespace, None)

        if app_context is None:
            error = self.cache_namespace_context(namespace=app_namespace, **kwargs)
            if error:
                return None
            with self._lock:
                app_context = self._namespace_contexts.get(app_namespace, None)

        if app_context is None or getattr(app_context, "type_", None) != "Game":
            return None

        studio_ns = getattr(app_context, "studio_namespace", None)
        publisher_ns = getattr(app_context, "publisher_namespace", None)

        with self._lock:
            if studio_ns:
                self._namespace_contexts.setdefault(
                    studio_ns,
                    NamespaceContext.create(
                        namespace=studio_ns,
                        type_="Studio",
                        studio_namespace=studio_ns,
                        publisher_namespace=publisher_ns,
                    ),
                )
            if publisher_ns:
                self._namespace_contexts.setdefault(
                    publisher_ns,
                    NamespaceContext.create(
                        namespace=publisher_ns,
                        type_="Publisher",
                        publisher_namespace=publisher_ns,
                    ),
                )
            return self._namespace_contexts.get(namespace, None)


class RevocationListCache(Timer):
    def __init__(
        self,
        sdk: AccelByteSDK,
        interval: Union[int, float],
        raise_on_error: bool = True,
        **kwargs,
    ):
        self.sdk = sdk
        self._revoked_token_filter: Optional[BloomFilter] = None
        self._revoked_users: Dict[str, float] = {}
        self._lock = RLock()
        self._raise_on_error = raise_on_error
        Timer.__init__(
            self,
            interval,
            self.update,
            daemon=True,
            repeats=-1,
            autostart=True,
            repeat_on_exception=True,
        )

    def update(self, **kwargs):
        result, error = iam_service.get_revocation_list_v3(
            x_additional_headers=kwargs.get("x_additional_headers", None),
            sdk=self.sdk,
        )
        if error:
            return

        with self._lock:
            # Revoked Tokens
            revoked_tokens = result.revoked_tokens
            self._revoked_token_filter = BloomFilter.create_from_bits(
                bits=revoked_tokens.bits, k=revoked_tokens.k, m=revoked_tokens.m
            )
            # Revoked Users
            revoked_users = result.revoked_users
            self._revoked_users = {}
            for user in revoked_users:
                if user.id_ and user.revoked_at:
                    revoked_at = str2datetime(user.revoked_at).timestamp()
                    self._revoked_users[user.id_] = revoked_at

    def is_token_revoked(self, token: str, **kwargs) -> bool:
        with self._lock:
            if self._revoked_token_filter:
                return self._revoked_token_filter.might_contains(key=token)
            else:
                return False

    def is_user_revoked(self, user_id: str, issued_at: int, **kwargs) -> bool:
        with self._lock:
            revoked_at = self._revoked_users.get(user_id, None)
            if revoked_at is not None:
                return revoked_at >= issued_at
        return False


class RolesCache(Timer):
    def __init__(
        self,
        sdk: AccelByteSDK,
        interval: Optional[Union[int, float]] = None,
        raise_on_error: bool = True,
        **kwargs,
    ):
        self.sdk = sdk
        self._roles: Dict[Tuple[str, Optional[str]], Any] = {}
        self._inflight: Dict[Tuple[str, Optional[str]], _InflightFetch] = {}
        self._lock = RLock()
        self._raise_on_error = raise_on_error
        self._generation: int = 0
        if interval is not None:
            Timer.__init__(
                self,
                interval,
                self.refresh,
                daemon=True,
                repeats=-1,
                autostart=True,
                repeat_on_exception=True,
            )

    def clear(self):
        """Drop all cached entries and pending single-flight coordinators.

        Bumps the generation counter so any concurrent refresh() aborts before
        re-populating the cleared cache. Signals waiters on outstanding
        coordinators before discarding them so no thread blocks indefinitely.
        Waiters woken this way observe inflight.error == None (the fetch had
        not yet completed) and surface a transient None / 401 to callers --
        the expected semantic for a deliberately invalidated cache.

        The fetcher's finally block may call event.set() again on a coordinator
        already signalled here; threading.Event.set() is idempotent.
        """
        with self._lock:
            self._generation += 1
            for fetch in self._inflight.values():
                fetch.event.set()
            self._roles = {}
            self._inflight = {}

    def update(self, **kwargs):
        pass

    def refresh(self, **kwargs):
        """Re-fetch every currently-cached role in place.

        On per-key failure the prior entry is retained (serve-stale), so a transient
        IAM outage cannot empty the cache and cascade into 401 InsufficientPermissions.

        Invoked by the background Timer with no kwargs, so x_additional_headers is
        not propagated during automatic refreshes; auth flows through sdk=self.sdk.
        Live callers entering get_role for a refreshed key hit the cached entry
        before the single-flight gate, so they never block on the in-progress
        refresh. Aborts immediately if clear() bumps the generation counter
        mid-loop, so external invalidation is not silently undone. Keys
        currently in-flight (in _inflight but not yet in _roles) are not
        included in the snapshot; they will be picked up next cycle.
        """
        with self._lock:
            gen = self._generation
            keys = list(self._roles.keys())
        for role_id, namespace in keys:
            with self._lock:
                if self._generation != gen:
                    return
            try:
                error = self.cache_role(role_id=role_id, namespace=namespace, **kwargs)
            except Exception as exc:
                logger.debug(
                    "role refresh raised for (%s, %s): %s; keeping stale entry",
                    role_id,
                    namespace,
                    exc,
                )
                continue
            if error:
                logger.debug(
                    "role refresh failed for (%s, %s): %s; keeping stale entry",
                    role_id,
                    namespace,
                    error,
                )
                continue
            # If clear() ran during the fetch, undo the spurious write so the
            # invalidation is not silently defeated by this refresh cycle.
            with self._lock:
                if self._generation != gen:
                    self._roles.pop((role_id, namespace), None)
                    return

    def cache_role(
        self, role_id: str, namespace: Optional[str] = None, **kwargs
    ) -> Any:
        cache_key = (role_id, namespace)

        with self._lock:
            inflight = self._inflight.get(cache_key)
            if inflight is None:
                inflight = _InflightFetch()
                self._inflight[cache_key] = inflight
                is_fetcher = True
            else:
                is_fetcher = False

        if not is_fetcher:
            logger.debug(
                "cache waiter: role=%s namespace=%s, waiting for in-flight fetch",
                role_id,
                namespace,
            )
            inflight.event.wait()
            err = inflight.error
            logger.debug(
                "cache waiter: role=%s namespace=%s, unblocked, error=%r",
                role_id,
                namespace,
                err,
            )
            if isinstance(err, BaseException):
                raise err
            return err

        try:
            try:
                role, error = iam_service.admin_get_role_namespace_permission_v3(
                    role_id=role_id,
                    namespace=namespace,
                    x_additional_headers=kwargs.get("x_additional_headers", None),
                    sdk=self.sdk,
                )
            except BaseException as exc:
                inflight.error = exc
                raise
            if error:
                inflight.error = error
                return error
            with self._lock:
                self._roles[cache_key] = role
            return None
        finally:
            with self._lock:
                self._inflight.pop(cache_key, None)
                inflight.event.set()

    def get_role(
        self, role_id: str, namespace: Optional[str] = None, **kwargs
    ) -> Optional[Role]:
        cache_key = (role_id, namespace)
        with self._lock:
            role = self._roles.get(cache_key, None)
            if role:
                return role

        # Lock released; concurrent callers for the same key are coalesced inside
        # cache_role via _InflightFetch. Do not remove that coordination without
        # also re-acquiring the lock around the fetch.
        error = self.cache_role(role_id=role_id, namespace=namespace, **kwargs)
        if error:
            if self._raise_on_error:
                raise FetchRoleError(f"failed to get role with ID: {role_id}")
            return None

        with self._lock:
            return self._roles.get(cache_key, None)

    def get_role_permissions(
        self, role_id: str, namespace: Optional[str] = None, **kwargs
    ) -> List[PermissionStruct]:
        role = self.get_role(role_id=role_id, namespace=namespace, **kwargs)
        role_permissions = getattr(role, "permissions", []) if role is not None else []
        return role_permissions

    def get_modified_role_permissions(
        self,
        role_id: str,
        namespace: str,
        user_id: Optional[str] = None,
        **kwargs,
    ) -> List[PermissionStruct]:
        role_permissions = self.get_role_permissions(
            role_id=role_id, namespace=namespace, **kwargs
        )
        modified_role_permissions = []
        for permission in role_permissions:
            action = getattr(permission, "action", None)
            resource = getattr(permission, "resource", None)
            if action is not None and resource is not None:
                permission_struct = create_permission_struct(
                    action=action,
                    resource=replace_resource(
                        resource=resource,
                        namespace=namespace,
                        user_id=user_id,
                    ),
                )
                modified_role_permissions.append(permission_struct)
        return modified_role_permissions

    def get_modified_role_permissions2(
        self,
        namespace_role: NamespaceRole,
        user_id: Optional[str] = None,
        **kwargs,
    ) -> List[PermissionStruct]:
        role_id = namespace_role.get("roleId", None)
        return (
            self.get_modified_role_permissions(
                role_id=role_id,
                namespace=namespace_role.get("namespace", None),
                user_id=user_id,
                **kwargs,
            )
            if role_id is not None
            else []
        )


__all__ = [
    # Type Aliases
    "JWTClaims",
    "JWKSet",
    "NamespaceContext",
    "NamespaceRole",
    "PublicPrivateKey",
    "Role",
    # Cache Types
    "JWKSCache",
    "NamespaceContextCache",
    "RevocationListCache",
    "RolesCache",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.apimodel_sql_database_credential_response import (
    ApimodelSQLDatabaseCredentialResponse,
)


class ApimodelSQLDatabaseResponse(Model):
    """Apimodel SQL database response (apimodel.SQLDatabaseResponse)

    Properties:
        accept_sql_secure_credential_handling: (acceptSQLSecureCredentialHandling) REQUIRED bool

        app_id: (appId) REQUIRED str

        credentials: (credentials) REQUIRED ApimodelSQLDatabaseCredentialResponse

        db_id: (dbId) REQUIRED str

        db_name: (dbName) REQUIRED str

        game_namespace: (gameNamespace) REQUIRED str

        hostnames: (hostnames) REQUIRED str

        platform_name: (platformName) REQUIRED str

        resource_id: (resourceId) REQUIRED str

        resource_status: (resourceStatus) REQUIRED str
    """

    # region fields

    accept_sql_secure_credential_handling: bool  # REQUIRED
    app_id: str  # REQUIRED
    credentials: ApimodelSQLDatabaseCredentialResponse  # REQUIRED
    db_id: str  # REQUIRED
    db_name: str  # REQUIRED
    game_namespace: str  # REQUIRED
    hostnames: str  # REQUIRED
    platform_name: str  # REQUIRED
    resource_id: str  # REQUIRED
    resource_status: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_accept_sql_secure_credential_handling(
        self, value: bool
    ) -> ApimodelSQLDatabaseResponse:
        self.accept_sql_secure_credential_handling = value
        return self

    def with_app_id(self, value: str) -> ApimodelSQLDatabaseResponse:
        self.app_id = value
        return self

    def with_credentials(
        self, value: ApimodelSQLDatabaseCredentialResponse
    ) -> ApimodelSQLDatabaseResponse:
        self.credentials = value
        return self

    def with_db_id(self, value: str) -> ApimodelSQLDatabaseResponse:
        self.db_id = value
        return self

    def with_db_name(self, value: str) -> ApimodelSQLDatabaseResponse:
        self.db_name = value
        return self

    def with_game_namespace(self, value: str) -> ApimodelSQLDatabaseResponse:
        self.game_namespace = value
        return self

    def with_hostnames(self, value: str) -> ApimodelSQLDatabaseResponse:
        self.hostnames = value
        return self

    def with_platform_name(self, value: str) -> ApimodelSQLDatabaseResponse:
        self.platform_name = value
        return self

    def with_resource_id(self, value: str) -> ApimodelSQLDatabaseResponse:
        self.resource_id = value
        return self

    def with_resource_status(self, value: str) -> ApimodelSQLDatabaseResponse:
        self.resource_status = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "accept_sql_secure_credential_handling"):
            result["acceptSQLSecureCredentialHandling"] = bool(
                self.accept_sql_secure_credential_handling
            )
        elif include_empty:
            result["acceptSQLSecureCredentialHandling"] = False
        if hasattr(self, "app_id"):
            result["appId"] = str(self.app_id)
        elif include_empty:
            result["appId"] = ""
        if hasattr(self, "credentials"):
            result["credentials"] = self.credentials.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["credentials"] = ApimodelSQLDatabaseCredentialResponse()
        if hasattr(self, "db_id"):
            result["dbId"] = str(self.db_id)
        elif include_empty:
            result["dbId"] = ""
        if hasattr(self, "db_name"):
            result["dbName"] = str(self.db_name)
        elif include_empty:
            result["dbName"] = ""
        if hasattr(self, "game_namespace"):
            result["gameNamespace"] = str(self.game_namespace)
        elif include_empty:
            result["gameNamespace"] = ""
        if hasattr(self, "hostnames"):
            result["hostnames"] = str(self.hostnames)
        elif include_empty:
            result["hostnames"] = ""
        if hasattr(self, "platform_name"):
            result["platformName"] = str(self.platform_name)
        elif include_empty:
            result["platformName"] = ""
        if hasattr(self, "resource_id"):
            result["resourceId"] = str(self.resource_id)
        elif include_empty:
            result["resourceId"] = ""
        if hasattr(self, "resource_status"):
            result["resourceStatus"] = str(self.resource_status)
        elif include_empty:
            result["resourceStatus"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        accept_sql_secure_credential_handling: bool,
        app_id: str,
        credentials: ApimodelSQLDatabaseCredentialResponse,
        db_id: str,
        db_name: str,
        game_namespace: str,
        hostnames: str,
        platform_name: str,
        resource_id: str,
        resource_status: str,
        **kwargs,
    ) -> ApimodelSQLDatabaseResponse:
        instance = cls()
        instance.accept_sql_secure_credential_handling = (
            accept_sql_secure_credential_handling
        )
        instance.app_id = app_id
        instance.credentials = credentials
        instance.db_id = db_id
        instance.db_name = db_name
        instance.game_namespace = game_namespace
        instance.hostnames = hostnames
        instance.platform_name = platform_name
        instance.resource_id = resource_id
        instance.resource_status = resource_status
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelSQLDatabaseResponse:
        instance = cls()
        if not dict_:
            return instance
        if (
            "acceptSQLSecureCredentialHandling" in dict_
            and dict_["acceptSQLSecureCredentialHandling"] is not None
        ):
            instance.accept_sql_secure_credential_handling = bool(
                dict_["acceptSQLSecureCredentialHandling"]
            )
        elif include_empty:
            instance.accept_sql_secure_credential_handling = False
        if "appId" in dict_ and dict_["appId"] is not None:
            instance.app_id = str(dict_["appId"])
        elif include_empty:
            instance.app_id = ""
        if "credentials" in dict_ and dict_["credentials"] is not None:
            instance.credentials = (
                ApimodelSQLDatabaseCredentialResponse.create_from_dict(
                    dict_["credentials"], include_empty=include_empty
                )
            )
        elif include_empty:
            instance.credentials = ApimodelSQLDatabaseCredentialResponse()
        if "dbId" in dict_ and dict_["dbId"] is not None:
            instance.db_id = str(dict_["dbId"])
        elif include_empty:
            instance.db_id = ""
        if "dbName" in dict_ and dict_["dbName"] is not None:
            instance.db_name = str(dict_["dbName"])
        elif include_empty:
            instance.db_name = ""
        if "gameNamespace" in dict_ and dict_["gameNamespace"] is not None:
            instance.game_namespace = str(dict_["gameNamespace"])
        elif include_empty:
            instance.game_namespace = ""
        if "hostnames" in dict_ and dict_["hostnames"] is not None:
            instance.hostnames = str(dict_["hostnames"])
        elif include_empty:
            instance.hostnames = ""
        if "platformName" in dict_ and dict_["platformName"] is not None:
            instance.platform_name = str(dict_["platformName"])
        elif include_empty:
            instance.platform_name = ""
        if "resourceId" in dict_ and dict_["resourceId"] is not None:
            instance.resource_id = str(dict_["resourceId"])
        elif include_empty:
            instance.resource_id = ""
        if "resourceStatus" in dict_ and dict_["resourceStatus"] is not None:
            instance.resource_status = str(dict_["resourceStatus"])
        elif include_empty:
            instance.resource_status = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelSQLDatabaseResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelSQLDatabaseResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelSQLDatabaseResponse,
        List[ApimodelSQLDatabaseResponse],
        Dict[Any, ApimodelSQLDatabaseResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "acceptSQLSecureCredentialHandling": "accept_sql_secure_credential_handling",
            "appId": "app_id",
            "credentials": "credentials",
            "dbId": "db_id",
            "dbName": "db_name",
            "gameNamespace": "game_namespace",
            "hostnames": "hostnames",
            "platformName": "platform_name",
            "resourceId": "resource_id",
            "resourceStatus": "resource_status",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "acceptSQLSecureCredentialHandling": True,
            "appId": True,
            "credentials": True,
            "dbId": True,
            "dbName": True,
            "gameNamespace": True,
            "hostnames": True,
            "platformName": True,
            "resourceId": True,
            "resourceStatus": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.platform_reward import PlatformReward


class DLCItem(Model):
    """DLC item (DLCItem)

    Properties:
        auto_update: (autoUpdate) OPTIONAL bool

        enable_revocation: (enableRevocation) OPTIONAL bool

        id_: (id) OPTIONAL str

        rewards: (rewards) OPTIONAL List[PlatformReward]

        rvn: (rvn) OPTIONAL int
    """

    # region fields

    auto_update: bool  # OPTIONAL
    enable_revocation: bool  # OPTIONAL
    id_: str  # OPTIONAL
    rewards: List[PlatformReward]  # OPTIONAL
    rvn: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_auto_update(self, value: bool) -> DLCItem:
        self.auto_update = value
        return self

    def with_enable_revocation(self, value: bool) -> DLCItem:
        self.enable_revocation = value
        return self

    def with_id(self, value: str) -> DLCItem:
        self.id_ = value
        return self

    def with_rewards(self, value: List[PlatformReward]) -> DLCItem:
        self.rewards = value
        return self

    def with_rvn(self, value: int) -> DLCItem:
        self.rvn = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "auto_update"):
            result["autoUpdate"] = bool(self.auto_update)
        elif include_empty:
            result["autoUpdate"] = False
        if hasattr(self, "enable_revocation"):
            result["enableRevocation"] = bool(self.enable_revocation)
        elif include_empty:
            result["enableRevocation"] = False
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "rewards"):
            result["rewards"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.rewards
            ]
        elif include_empty:
            result["rewards"] = []
        if hasattr(self, "rvn"):
            result["rvn"] = int(self.rvn)
        elif include_empty:
            result["rvn"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        auto_update: Optional[bool] = None,
        enable_revocation: Optional[bool] = None,
        id_: Optional[str] = None,
        rewards: Optional[List[PlatformReward]] = None,
        rvn: Optional[int] = None,
        **kwargs,
    ) -> DLCItem:
        instance = cls()
        if auto_update is not None:
            instance.auto_update = auto_update
        if enable_revocation is not None:
            instance.enable_revocation = enable_revocation
        if id_ is not None:
            instance.id_ = id_
        if rewards is not None:
            instance.rewards = rewards
        if rvn is not None:
            instance.rvn = rvn
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> DLCItem:
        instance = cls()
        if not dict_:
            return instance
        if "autoUpdate" in dict_ and dict_["autoUpdate"] is not None:
            instance.auto_update = bool(dict_["autoUpdate"])
        elif include_empty:
            instance.auto_update = False
        if "enableRevocation" in dict_ and dict_["enableRevocation"] is not None:
            instance.enable_revocation = bool(dict_["enableRevocation"])
        elif include_empty:
            instance.enable_revocation = False
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "rewards" in dict_ and dict_["rewards"] is not None:
            instance.rewards = [
                PlatformReward.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["rewards"]
            ]
        elif include_empty:
            instance.rewards = []
        if "rvn" in dict_ and dict_["rvn"] is not None:
            instance.rvn = int(dict_["rvn"])
        elif include_empty:
            instance.rvn = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, DLCItem]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[DLCItem]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[DLCItem, List[DLCItem], Dict[Any, DLCItem]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "autoUpdate": "auto_update",
            "enableRevocation": "enable_revocation",
            "id": "id_",
            "rewards": "rewards",
            "rvn": "rvn",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "autoUpdate": False,
            "enableRevocation": False,
            "id": False,
            "rewards": False,
            "rvn": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ApimodelCSMAppLimitsResponse
from ..models import ResponseErrorResponse

from ..operations.resources_limits import GetResourcesLimits


@same_doc_as(GetResourcesLimits)
def get_resources_limits(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Extend Apps Configurable Limits e.g. any kind of limits for front end validation use (GetResourcesLimits)

    This endpoint return the Extend Apps Configurable Limits for front end to use for initial validation before app creation
    e.g.
    Replica Limit will be used to check max replica that can be created for the said environment by default
    CPU Limit will be used to validate max allowed CPU for the extend app that hasn't been created
    Memory Limit will be used to validate max allowed Memory for the extend app that hasn't been created
    MaxAppNotificationSubscription will be used to validate the maximum number of subscriber for an app status notification

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/resources/limits

        method: GET

        tags: ["Resources Limits"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelCSMAppLimitsResponse

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        402: Payment Required - ResponseErrorResponse (Payment Required)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetResourcesLimits.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetResourcesLimits)
async def get_resources_limits_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Extend Apps Configurable Limits e.g. any kind of limits for front end validation use (GetResourcesLimits)

    This endpoint return the Extend Apps Configurable Limits for front end to use for initial validation before app creation
    e.g.
    Replica Limit will be used to check max replica that can be created for the said environment by default
    CPU Limit will be used to validate max allowed CPU for the extend app that hasn't been created
    Memory Limit will be used to validate max allowed Memory for the extend app that hasn't been created
    MaxAppNotificationSubscription will be used to validate the maximum number of subscriber for an app status notification

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/resources/limits

        method: GET

        tags: ["Resources Limits"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelCSMAppLimitsResponse

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        402: Payment Required - ResponseErrorResponse (Payment Required)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetResourcesLimits.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Cloudsave Service."""

__version__ = "3.32.2"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_delete_player_bin_892eb5 import AdminDeletePlayerBinaryRecordV1
from .admin_get_player_binary_1076b9 import AdminGetPlayerBinaryRecordV1
from .admin_list_player_binar_e452ce import AdminListPlayerBinaryRecordsV1
from .admin_post_player_binar_f1a4e9 import AdminPostPlayerBinaryPresignedURLV1
from .admin_post_player_binar_e33f40 import AdminPostPlayerBinaryRecordV1
from .admin_put_player_binary_bcf941 import AdminPutPlayerBinaryRecorMetadataV1
from .admin_put_player_binary_6424e4 import AdminPutPlayerBinaryRecordV1
